@@ -2,15 +2,15 @@
 
   dln.c -
 
-  $Author: knu $
-  $Date: 2008-06-06 19:39:57 +0900 (Fri, 06 Jun 2008) $
+  $Author: matz $
+  $Date: 2007-08-25 12:29:39 +0900 (Sat, 25 Aug 2007) $
   created at: Tue Jan 18 17:05:06 JST 1994
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
+#include "ruby/ruby.h"
 #include "dln.h"
 
 #ifdef HAVE_STDLIB_H
@@ -110,9 +110,7 @@ char *getenv();
 #endif
 
 static int
-init_funcname_len(buf, file)
-    char **buf;
-    const char *file;
+init_funcname_len(char **buf, const char *file)
 {
     char *p;
     const char *slash;
@@ -184,8 +182,8 @@ static int dln_init_p = 0;
 
 #define INVALID_OBJECT(h) (N_MAGIC(h) != OMAGIC)
 
-#include "util.h"
-#include "st.h"
+#include "ruby/util.h"
+#include "ruby/st.h"
 
 static st_table *sym_tbl;
 static st_table *undef_tbl;
@@ -193,10 +191,7 @@ static st_table *undef_tbl;
 static int load_lib();
 
 static int
-load_header(fd, hdrp, disp)
-    int fd;
-    struct exec *hdrp;
-    long disp;
+load_header(int fd, struct exec *hdrp, long disp)
 {
     int size;
 
@@ -266,10 +261,7 @@ static int reloc_r_length[] = {
 #endif
 
 static struct relocation_info *
-load_reloc(fd, hdrp, disp)
-     int fd;
-     struct exec *hdrp;
-     long disp;
+load_reloc(int fd, struct exec *hdrp, long disp)
 {
     struct relocation_info *reloc;
     int size;
@@ -292,10 +284,7 @@ load_reloc(fd, hdrp, disp)
 }
 
 static struct nlist *
-load_sym(fd, hdrp, disp)
-    int fd;
-    struct exec *hdrp;
-    long disp;
+load_sym(int fd, struct exec *hdrp, long disp)
 {
     struct nlist * buffer;
     struct nlist * sym;
@@ -336,9 +325,7 @@ load_sym(fd, hdrp, disp)
 }
 
 static st_table *
-sym_hash(hdrp, syms)
-    struct exec *hdrp;
-    struct nlist *syms;
+sym_hash(struct exec *hdrp, struct nlist *syms)
 {
     st_table *tbl;
     struct nlist *sym = syms;
@@ -358,8 +345,7 @@ sym_hash(hdrp, syms)
 }
 
 static int
-dln_init(prog)
-    const char *prog;
+dln_init(const char *prog)
 {
     char *file;
     int fd;
@@ -433,11 +419,7 @@ dln_init(prog)
 }
 
 static long
-load_text_data(fd, hdrp, bss, disp)
-    int fd;
-    struct exec *hdrp;
-    int bss;
-    long disp;
+load_text_data(int fd, struct exec *hdrp, int bss, long disp)
 {
     int size;
     unsigned char* addr;
@@ -471,8 +453,7 @@ load_text_data(fd, hdrp, bss, disp)
 }
 
 static int
-undef_print(key, value)
-    char *key, *value;
+undef_print(char *key, char *value)
 {
     fprintf(stderr, "  %s\n", key);
     return ST_CONTINUE;
@@ -509,10 +490,7 @@ struct undef {
 
 static st_table *reloc_tbl = NULL;
 static void
-link_undef(name, base, reloc)
-    const char *name;
-    long base;
-    struct relocation_info *reloc;
+link_undef(const char *name, long base, struct relocation_info *reloc)
 {
     static int u_no = 0;
     struct undef *obj;
@@ -545,10 +523,7 @@ struct reloc_arg {
 };
 
 static int
-reloc_undef(no, undef, arg)
-    int no;
-    struct undef *undef;
-    struct reloc_arg *arg;
+reloc_undef(int no, struct undef *undef, struct reloc_arg *arg)
 {
     int datum;
     char *address;
@@ -609,9 +584,7 @@ reloc_undef(no, undef, arg)
 }
 
 static void
-unlink_undef(name, value)
-    const char *name;
-    long value;
+unlink_undef(const char *name, long value)
 {
     struct reloc_arg arg;
 
@@ -626,10 +599,7 @@ struct indr_data {
 };
 
 static int
-reloc_repl(no, undef, data)
-    int no;
-    struct undef *undef;
-    struct indr_data *data;
+reloc_repl(int no, struct undef *undef, struct indr_data *data)
 {
     if (strcmp(data->name0, undef->name) == 0) {
 	free(undef->name);
@@ -640,10 +610,7 @@ reloc_repl(no, undef, data)
 #endif
 
 static int
-load_1(fd, disp, need_init)
-    int fd;
-    long disp;
-    const char *need_init;
+load_1(int fd, long disp, const char *need_init)
 {
     static const char *libc = LIBC_NAME;
     struct exec hdr;
@@ -924,10 +891,7 @@ load_1(fd, disp, need_init)
 
 static int target_offset;
 static int
-search_undef(key, value, lib_tbl)
-    const char *key;
-    int value;
-    st_table *lib_tbl;
+search_undef(const char *key, int value, st_table *lib_tbl)
 {
     long offset;
 
@@ -944,8 +908,7 @@ struct symdef {
 char *dln_librrb_ary_path = DLN_DEFAULT_LIB_PATH;
 
 static int
-load_lib(lib)
-    const char *lib;
+load_lib(const char *lib)
 {
     char *path, *file;
     char armagic[SARMAG];
@@ -1082,8 +1045,7 @@ load_lib(lib)
 }
 
 static int
-load(file)
-    const char *file;
+load(const char *file)
 {
     int fd;
     int result;
@@ -1108,8 +1070,7 @@ load(file)
 }
 
 void*
-dln_sym(name)
-    const char *name;
+dln_sym(const char *name)
 {
     struct nlist *sym;
 
@@ -1121,12 +1082,7 @@ dln_sym(name)
 #endif /* USE_DLN_A_OUT */
 
 #ifdef USE_DLN_DLOPEN
-# if defined(__NetBSD__) && defined(__NetBSD_Version__) && __NetBSD_Version__ < 105000000
-#  include <nlist.h>
-#  include <link.h>
-# else
-#  include <dlfcn.h>
-# endif
+# include <dlfcn.h>
 #endif
 
 #ifdef __hpux
@@ -1169,7 +1125,7 @@ dln_sym(name)
 #endif
 
 static const char *
-dln_strerror()
+dln_strerror(void)
 {
 #ifdef USE_DLN_A_OUT
     char *strerror();
@@ -1267,24 +1223,10 @@ aix_loaderror(const char *pathname)
 }
 #endif
 
-#if defined(__VMS)
-#include <starlet.h>
-#include <rms.h>
-#include <stsdef.h>
-#include <unixlib.h>
-#include <descrip.h>
-#include <lib$routines.h>
-
-static char *vms_filespec;
-static int vms_fileact(char *filespec, int type);
-static long vms_fisexh(long *sigarr, long *mecarr);
-#endif
-
 #endif /* NO_DLN_LOAD */
 
 void*
-dln_load(file)
-    const char *file;
+dln_load(const char *file)
 {
 #ifdef NO_DLN_LOAD
     rb_raise(rb_eLoadError, "this executable file can't load extension libraries");
@@ -1586,16 +1528,9 @@ dln_load(file)
 #if defined(__VMS)
 #define DLN_DEFINED
     {
-	long status;
-	void (*init_fct)();
+	void *handle, (*init_fct)();
 	char *fname, *p1, *p2;
 
-	$DESCRIPTOR(fname_d, "");
-	$DESCRIPTOR(image_d, "");
-	$DESCRIPTOR(buf_d, "");
-
-	decc$to_vms(file, vms_fileact, 0, 0);
-
 	fname = (char *)__alloca(strlen(file)+1);
 	strcpy(fname,file);
 	if (p1 = strrchr(fname,'/'))
@@ -1603,35 +1538,19 @@ dln_load(file)
 	if (p2 = strrchr(fname,'.'))
 	    *p2 = '\0';
 
-	fname_d.dsc$w_length  = strlen(fname);
-	fname_d.dsc$a_pointer = fname;
-	image_d.dsc$w_length  = strlen(vms_filespec);
-	image_d.dsc$a_pointer = vms_filespec;
-	buf_d.dsc$w_length    = strlen(buf);
-	buf_d.dsc$a_pointer   = buf;
-
-	lib$establish(vms_fisexh);
-
-	status = lib$find_image_symbol (
-		     &fname_d,
-		     &buf_d, 
-		     &init_fct, 
-		     &image_d);
-
-	lib$establish(0);
-
-	if (status == RMS$_FNF) {
+	if ((handle = (void*)dlopen(fname, 0)) == NULL) {
 	    error = dln_strerror();
 	    goto failed;
-	} else if (!$VMS_STATUS_SUCCESS(status)) {
-	    error = DLN_ERROR();
-	    goto failed;
 	}
 
+	if ((init_fct = (void (*)())dlsym(handle, buf)) == NULL) {
+	    error = DLN_ERROR();
+	    dlclose(handle);
+	    goto failed;
+	}
 	/* Call the init code */
 	(*init_fct)();
-
-	return 1;
+	return handle;
     }
 #endif /* __VMS */
 
@@ -1650,12 +1569,10 @@ dln_load(file)
     return 0;			/* dummy return */
 }
 
-static char *dln_find_1();
+static char *dln_find_1(const char *fname, const char *path, int exe_flag);
 
 char *
-dln_find_exe(fname, path)
-    const char *fname;
-    const char *path;
+dln_find_exe(const char *fname, const char *path)
 {
     if (!path) {
 	path = getenv(PATH_ENV);
@@ -1672,9 +1589,7 @@ dln_find_exe(fname, path)
 }
 
 char *
-dln_find_file(fname, path)
-    const char *fname;
-    const char *path;
+dln_find_file(const char *fname, const char *path)
 {
 #ifndef __MACOS__
     if (!path) path = ".";
@@ -1688,10 +1603,7 @@ dln_find_file(fname, path)
 static char fbuf[MAXPATHLEN];
 
 static char *
-dln_find_1(fname, path, exe_flag)
-    const char *fname;
-    const char *path;
-    int exe_flag;		/* non 0 if looking for executable. */
+dln_find_1(const char *fname, const char *path, int exe_flag /* non 0 if looking for executable. */)
 {
     register const char *dp;
     register const char *ep;
@@ -1701,21 +1613,23 @@ dln_find_1(fname, path, exe_flag)
     const char* mac_fullpath;
 #endif
 
-    if (!fname) return (char *)fname;
-    if (fname[0] == '/') return (char *)fname;
-    if (strncmp("./", fname, 2) == 0 || strncmp("../", fname, 3) == 0)
-      return (char *)fname;
-    if (exe_flag && strchr(fname, '/')) return (char *)fname;
+#define RETURN_IF(expr) if (expr) return (char *)fname;
+
+    RETURN_IF(!fname);
+    RETURN_IF(fname[0] == '/');
+    RETURN_IF(strncmp("./", fname, 2) == 0 || strncmp("../", fname, 3) == 0);
+    RETURN_IF(exe_flag && strchr(fname, '/'));
 #ifdef DOSISH
-    if (fname[0] == '\\') return (char *)fname;
+    RETURN_IF(fname[0] == '\\');
 # ifdef DOSISH_DRIVE_LETTER
-    if (strlen(fname) > 2 && fname[1] == ':') return (char *)fname;
+    RETURN_IF(strlen(fname) > 2 && fname[1] == ':');
 # endif
-    if (strncmp(".\\", fname, 2) == 0 || strncmp("..\\", fname, 3) == 0)
-      return (char *)fname;
-    if (exe_flag && strchr(fname, '\\')) return (char *)fname;
+    RETURN_IF(strncmp(".\\", fname, 2) == 0 || strncmp("..\\", fname, 3) == 0);
+    RETURN_IF(exe_flag && strchr(fname, '\\'));
 #endif
 
+#undef RETURN_IF
+
     for (dp = path;; dp = ++ep) {
 	register int l;
 	int i;
@@ -1781,45 +1695,6 @@ dln_find_1(fname, path, exe_flag)
 	}
 	memcpy(bp, fname, i + 1);
 
-#if defined(DOSISH)
-	if (exe_flag) {
-	    static const char extension[][5] = {
-#if defined(MSDOS)
-		".com", ".exe", ".bat",
-#if defined(DJGPP)
-		".btm", ".sh", ".ksh", ".pl", ".sed",
-#endif
-#elif defined(__EMX__) || defined(_WIN32)
-		".exe", ".com", ".cmd", ".bat",
-/* end of __EMX__ or _WIN32 */
-#else
-		".r", ".R", ".x", ".X", ".bat", ".BAT",
-/* __human68k__ */
-#endif
-	    };
-	    int j;
-
-	    for (j = 0; j < sizeof(extension) / sizeof(extension[0]); j++) {
-		if (fspace < strlen(extension[j])) {
-		    fprintf(stderr, "openpath: pathname too long (ignored)\n");
-		    fprintf(stderr, "\tDirectory \"%.*s\"\n", (int) (bp - fbuf), fbuf);
-		    fprintf(stderr, "\tFile \"%s%s\"\n", fname, extension[j]);
-		    continue;
-		}
-		strcpy(bp + i, extension[j]);
-#ifndef __MACOS__
-		if (stat(fbuf, &st) == 0)
-		    return fbuf;
-#else
-		if (mac_fullpath = _macruby_exist_file_in_libdir_as_posix_name(fbuf))
-		    return mac_fullpath;
-
-#endif
-	    }
-	    goto next;
-	}
-#endif /* MSDOS or _WIN32 or __human68k__ or __EMX__ */
-
 #ifndef __MACOS__
 	if (stat(fbuf, &st) == 0) {
 	    if (exe_flag == 0) return fbuf;
@@ -1837,6 +1712,44 @@ dln_find_1(fname, path, exe_flag)
 	    }
 	}
 #endif
+#if defined(DOSISH)
+	if (exe_flag) {
+	    static const char *const extension[] = {
+#if defined(MSDOS)
+		".com", ".exe", ".bat",
+#if defined(DJGPP)
+		".btm", ".sh", ".ksh", ".pl", ".sed",
+#endif
+#elif defined(__EMX__) || defined(_WIN32)
+		".exe", ".com", ".cmd", ".bat",
+/* end of __EMX__ or _WIN32 */
+#else
+		".r", ".R", ".x", ".X", ".bat", ".BAT",
+/* __human68k__ */
+#endif
+		(char *) NULL
+	    };
+	    int j;
+
+	    for (j = 0; extension[j]; j++) {
+		if (fspace < strlen(extension[j])) {
+		    fprintf(stderr, "openpath: pathname too long (ignored)\n");
+		    fprintf(stderr, "\tDirectory \"%.*s\"\n", (int) (bp - fbuf), fbuf);
+		    fprintf(stderr, "\tFile \"%s%s\"\n", fname, extension[j]);
+		    continue;
+		}
+		strcpy(bp + i, extension[j]);
+#ifndef __MACOS__
+		if (stat(fbuf, &st) == 0)
+		    return fbuf;
+#else
+		if (mac_fullpath = _macruby_exist_file_in_libdir_as_posix_name(fbuf))
+		    return mac_fullpath;
+
+#endif
+	    }
+	}
+#endif /* MSDOS or _WIN32 or __human68k__ or __EMX__ */
 
       next:
 	/* if not, and no other alternatives, life is bleak */
@@ -1847,24 +1760,3 @@ dln_find_1(fname, path, exe_flag)
 	/* otherwise try the next component in the search path */
     }
 }
-
-#if defined(__VMS)
-
-/* action routine for decc$to_vms */
-static int vms_fileact(char *filespec, int type)
-{
-    if (vms_filespec)
-	free(vms_filespec);
-    vms_filespec = malloc(strlen(filespec)+1);
-    strcpy(vms_filespec, filespec);
-    return 1;
-}
-
-/* exception handler for LIB$FIND_IMAGE_SYMBOL */
-static long vms_fisexh(long *sigarr, long *mecarr)
-{
-    sys$unwind(1, 0);
-    return 1;
-}
-
-#endif /* __VMS */
