@@ -42,15 +42,6 @@ def sysquote(x)
   @quote ? x.quote : x
 end
 
-def relative_from(path, base)
-  dir = File.join(path, "")
-  if File.expand_path(dir) == File.expand_path(dir, base)
-    path
-  else
-    File.join(base, path)
-  end
-end
-
 def extract_makefile(makefile, keep = true)
   m = File.read(makefile)
   if !(target = m[/^TARGET[ \t]*=[ \t]*(\S*)/, 1])
@@ -69,7 +60,7 @@ def extract_makefile(makefile, keep = true)
       unless installrb.empty?
         config = CONFIG.dup
         install_dirs(target_prefix).each {|var, val| config[var] = val}
-        FileUtils.rm_f(installrb.values.collect {|f| Config.expand(f, config)}, :verbose => true)
+        FileUtils.rm_f(installrb.values.collect {|f| RbConfig.expand(f, config)}, verbose: true)
       end
     end
     return false
@@ -127,7 +118,7 @@ def extmake(target)
     makefile = "./Makefile"
     ok = File.exist?(makefile)
     unless $ignore
-      rbconfig0 = Config::CONFIG
+      rbconfig0 = RbConfig::CONFIG
       mkconfig0 = CONFIG
       rbconfig = {
 	"hdrdir" => $hdrdir,
@@ -135,13 +126,13 @@ def extmake(target)
 	"topdir" => $topdir,
       }
       mkconfig = {
-	"hdrdir" => "$(top_srcdir)",
+	"hdrdir" => ($hdrdir == top_srcdir) ? top_srcdir : "$(top_srcdir)/include",
 	"srcdir" => "$(top_srcdir)/ext/#{$mdir}",
 	"topdir" => $topdir,
       }
       rbconfig0.each_pair {|key, val| rbconfig[key] ||= val.dup}
       mkconfig0.each_pair {|key, val| mkconfig[key] ||= val.dup}
-      Config.module_eval {
+      RbConfig.module_eval {
 	remove_const(:CONFIG)
 	const_set(:CONFIG, rbconfig)
 	remove_const(:MAKEFILE_CONFIG)
@@ -154,17 +145,18 @@ def extmake(target)
       begin
 	$extconf_h = nil
 	ok &&= extract_makefile(makefile)
-	conf = ["#{$srcdir}/makefile.rb", "#{$srcdir}/extconf.rb"].find {|f| File.exist?(f)}
 	if (($extconf_h && !File.exist?($extconf_h)) ||
 	    !(t = modified?(makefile, MTIMES)) ||
-	    [conf, "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
+	    ["#{$srcdir}/makefile.rb", "#{$srcdir}/extconf.rb", "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
         then
 	  ok = false
           init_mkmf
 	  Logging::logfile 'mkmf.log'
 	  rm_f makefile
-	  if conf
-	    load $0 = conf
+	  if File.exist?($0 = "#{$srcdir}/makefile.rb")
+	    load $0
+	  elsif File.exist?($0 = "#{$srcdir}/extconf.rb")
+	    load $0
 	  else
 	    create_makefile(target)
 	  end
@@ -182,7 +174,7 @@ def extmake(target)
     ok = yield(ok) if block_given?
     unless ok
       open(makefile, "w") do |f|
-	f.print dummy_makefile(CONFIG["srcdir"])
+	f.print(*dummy_makefile(CONFIG["srcdir"]))
       end
       return true
     end
@@ -203,7 +195,6 @@ def extmake(target)
       if $clean != true
 	FileUtils.rm_f([makefile, $extconf_h || "extconf.h"])
       end
-      File.unlink(makefile) rescue nil
     end
     if $static
       $extflags ||= ""
@@ -217,7 +208,7 @@ def extmake(target)
     end
   ensure
     unless $ignore
-      Config.module_eval {
+      RbConfig.module_eval {
 	remove_const(:CONFIG)
 	const_set(:CONFIG, rbconfig0)
 	remove_const(:MAKEFILE_CONFIG)
@@ -249,7 +240,6 @@ end
 def parse_args()
   $mflags = []
 
-  opts = nil
   $optparser ||= OptionParser.new do |opts|
     opts.on('-n') {$dryrun = true}
     opts.on('--[no-]extension [EXTS]', Array) do |v|
@@ -288,7 +278,7 @@ def parse_args()
   rescue OptionParser::InvalidOption => e
     retry if /^--/ =~ e.args[0]
     $optparser.warn(e)
-    abort opts.to_s
+    abort $optparser.to_s
   end
 
   $destdir ||= ''
@@ -297,7 +287,7 @@ def parse_args()
   $mflags.unshift(*rest) unless rest.empty?
 
   def $mflags.set?(flag)
-    grep(/\A-(?!-).*#{flag.chr}/i) { return true }
+    grep(/\A-(?!-).*#{'%s' % flag}/i) { return true }
     false
   end
   def $mflags.defined?(var)
@@ -314,7 +304,7 @@ def parse_args()
   $continue = $mflags.set?(?k)
   if $extout
     $extout = '$(topdir)/'+$extout
-    Config::CONFIG["extout"] = CONFIG["extout"] = $extout
+    RbConfig::CONFIG["extout"] = CONFIG["extout"] = $extout
     $extout_prefix = $extout ? "$(extout)$(target_prefix)/" : ""
     $mflags << "extout=#$extout" << "extout_prefix=#$extout_prefix"
   end
@@ -353,23 +343,20 @@ elsif sep = config_string('BUILD_FILE_SEPARATOR')
 else
   $ruby = '$(topdir)/miniruby' + EXEEXT
 end
-$ruby << " -I'$(topdir)'"
-unless CROSS_COMPILING
-  $ruby << " -I'$(top_srcdir)/lib'"
-  $ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
-  $ruby << " -I./- -I'$(top_srcdir)/ext' -rpurelib.rb"
-  ENV["RUBYLIB"] = "-"
-  ENV["RUBYOPT"] = "-rpurelib.rb"
-end
-$config_h = '$(topdir)/config.h'
-$mflags << "ruby=#$ruby"
+$ruby << " -I'$(topdir)' -I'$(top_srcdir)/lib'"
+$ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
+$ruby << " -I'$(top_srcdir)/ext' -rpurelib.rb"
+$config_h = '$(arch_hdrdir)/ruby/config.h'
+ENV["RUBYLIB"] = "-"
+ENV["RUBYOPT"] = "-rpurelib.rb"
 
 MTIMES = [__FILE__, 'rbconfig.rb', srcdir+'/lib/mkmf.rb'].collect {|f| File.mtime(f)}
 
 # get static-link modules
 $static_ext = {}
 if $extstatic
-  $extstatic.each do |target|
+  $extstatic.each do |t|
+    target = t
     target = target.downcase if /mswin32|bccwin32/ =~ RUBY_PLATFORM
     $static_ext[target] = $static_ext.size
   end
@@ -406,17 +393,17 @@ if $extension
   exts |= $extension.select {|d| File.directory?("#{ext_prefix}/#{d}")}
 else
   withes, withouts = %w[--with --without].collect {|w|
-    if not (w = %w[-extensions -ext].collect {|opt|arg_config(w+opt)}).any?
+    if not (w = %w[-extensions -ext].collect {|o|arg_config(w+o)}).any?
       proc {false}
     elsif (w = w.grep(String)).empty?
       proc {true}
     else
-      proc {|c1| w.collect {|opt| opt.split(/,/)}.flatten.any?(&c1)}
+      w.collect {|o| o.split(/,/)}.flatten.method(:any?)
     end
   }
   cond = proc {|ext|
     cond1 = proc {|n| File.fnmatch(n, ext, File::FNM_PATHNAME)}
-    withes.call(cond1) or !withouts.call(cond1)
+    withes.call(&cond1) or !withouts.call(&cond1)
   }
   exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
     d = File.dirname(d)
@@ -428,7 +415,7 @@ else
 end
 
 if $extout
-  extout = Config.expand("#{$extout}", Config::CONFIG.merge("topdir"=>$topdir))
+  extout = RbConfig.expand("#{$extout}", RbConfig::CONFIG.merge("topdir"=>$topdir))
   unless $ignore
     FileUtils.mkpath(extout)
   end
@@ -439,7 +426,7 @@ FileUtils::makedirs('ext')
 Dir::chdir('ext')
 
 hdrdir = $hdrdir
-$hdrdir = $top_srcdir = relative_from(srcdir, $topdir = "..")
+$hdrdir = ($top_srcdir = relative_from(srcdir, $topdir = "..")) + "/include"
 exts.each do |d|
   extmake(d) or abort
 end
@@ -500,14 +487,17 @@ unless $extlist.empty?
   src = %{\
 #include "ruby.h"
 
-#define init(func, name) {void func _((void)); ruby_init_ext(name, func);}
+#define init(func, name) {	\\
+    extern void func _((void));	\\
+    ruby_init_ext(name, func);	\\
+}
 
 void ruby_init_ext _((const char *name, void (*init)(void)));
 
 void Init_ext _((void))\n{\n#$extinit}
 }
   if !modified?(extinit.c, MTIMES) || IO.read(extinit.c) != src
-    open(extinit.c, "w") {|f| f.print src}
+    open(extinit.c, "w") {|fe| fe.print src}
   end
 
   $extobjs = "ext/#{extinit.o} #{$extobjs}"
@@ -524,15 +514,15 @@ void Init_ext _((void))\n{\n#$extinit}
   ].map {|n, v|
     "#{n}=#{v}" if v and !(v = v.strip).empty?
   }.compact
-  puts conf
+  puts(*conf)
   $stdout.flush
   $mflags.concat(conf)
 else
   FileUtils.rm_f(extinit.to_a)
 end
 rubies = []
-%w[RUBY RUBYW STATIC_RUBY].each {|r|
-  n = r
+%w[RUBY RUBYW STATIC_RUBY].each {|n|
+  r = n
   if r = arg_config("--"+r.downcase) || config_string(r+"_INSTALL_NAME")
     rubies << Config.expand(r+=EXEEXT)
     $mflags << "#{n}=#{r}"
@@ -559,6 +549,7 @@ if $nmake == ?b
     vars.each {|flag| flag.sub!(/\A/, "-D")}
   end
 end
+$mflags.unshift("topdir=#$topdir")
 system($make, *sysquote($mflags)) or exit($?.exitstatus)
 
 #Local variables:
