@@ -1,22 +1,14 @@
 require 'mkmf'
 
 case RUBY_PLATFORM
-when /bccwin32/
+when /(ms|bcc)win32|mingw/
   test_func = "WSACleanup"
   have_library("ws2_32", "WSACleanup")
-when /mswin32|mingw/
-  test_func = "WSACleanup"
-  if with_config("winsock2")
-    have_library("ws2_32", "WSACleanup")
-  else
-    have_library("wsock32", "WSACleanup")
-  end
 when /cygwin/
   test_func = "socket"
 when /beos/
   test_func = "socket"
   have_library("net", "socket")
-  have_func("closesocket")
 when /i386-os2_emx/
   test_func = "socket"
   have_library("socket", "socket")
@@ -43,7 +35,6 @@ if enable_config("ipv6", default_ipv6)
   if checking_for("ipv6") {try_link(<<EOF)}
 #include <sys/types.h>
 #include <sys/socket.h>
-int
 main()
 {
   socket(AF_INET6, SOCK_STREAM, 0);
@@ -95,17 +86,10 @@ if have_struct_member("struct sockaddr_in", "sin_len", headers)
 end
 
 #   doug's fix, NOW add -Dss_family... only if required!
-[nil, " -Dss_family=__ss_family -Dss_len=__ss_len"].each do |flags|
-  if flags
-    cppflags = $CPPFLAGS
-    $CPPFLAGS += flags
-  end
-  if have_struct_member("struct sockaddr_storage", "ss_family", headers)
-    $defs[-1] = "-DHAVE_SOCKADDR_STORAGE"
-    break
-  elsif flags
-    $CPPFLAGS = cppflags
-  end
+doug = proc {have_struct_member("struct sockaddr_storage", "ss_family", headers)}
+if (doug[] or
+    with_cppflags($CPPFLAGS + " -Dss_family=__ss_family -Dss_len=__ss_len", &doug))
+  $defs[-1] = "-DHAVE_SOCKADDR_STORAGE"
 end
 
 if have_struct_member("struct sockaddr", "sa_len", headers)
@@ -120,9 +104,8 @@ if have_func("sendmsg") | have_func("recvmsg")
   have_struct_member('struct msghdr', 'msg_accrights', ['sys/types.h', 'sys/socket.h'])
 end
 
-getaddr_info_ok = (enable_config("wide-getaddrinfo") && :wide) ||
-  (checking_for("wide getaddrinfo") {try_run(<<EOF)} && :os)
-#{COMMON_HEADERS}
+getaddr_info_ok = enable_config("wide-getaddrinfo") do
+  checking_for("wide getaddrinfo") {try_run(<<EOF)}
 #{cpp_include(headers)}
 #include <stdlib.h>
 
@@ -137,17 +120,11 @@ getaddr_info_ok = (enable_config("wide-getaddrinfo") && :wide) ||
 #define AF_LOCAL AF_UNIX
 #endif
 
-int
 main()
 {
   int passive, gaierr, inet4 = 0, inet6 = 0;
   struct addrinfo hints, *ai, *aitop;
   char straddr[INET6_ADDRSTRLEN], strport[16];
-#ifdef _WIN32
-  WSADATA retdata;
-
-  WSAStartup(MAKEWORD(2, 0), &retdata);
-#endif
 
   for (passive = 0; passive <= 1; passive++) {
     memset(&hints, 0, sizeof(hints));
@@ -230,6 +207,7 @@ main()
   exit(EXIT_FAILURE);
 }
 EOF
+end
 if ipv6 and not getaddr_info_ok
   abort <<EOS
 
@@ -254,33 +232,11 @@ Fatal: invalid value for --with-lookup-order-hack (expected INET, INET6 or UNSPE
 EOS
 end
 
-have_type("struct addrinfo", headers)
-have_func("freehostent")
-have_func("freeaddrinfo")
-if have_func("gai_strerror")
-  if checking_for("gai_strerror() returns const pointer") {!try_compile(<<EOF)}
-#{COMMON_HEADERS}
-#{cpp_include(headers)}
-#include <stdlib.h>
-void
-conftest_gai_strerror_is_const()
-{
-    *gai_strerror(0) = 0;
-}
-EOF
-    $defs << "-DGAI_STRERROR_CONST"
-  end
-end
-
 $objs = ["socket.#{$OBJEXT}"]
 
-if getaddr_info_ok == :wide or
-    !have_func("getnameinfo", headers) or !have_func("getaddrinfo", headers)
+unless getaddr_info_ok and have_func("getnameinfo", "netdb.h") and have_func("getaddrinfo", "netdb.h")
   if have_struct_member("struct in6_addr", "s6_addr8", headers)
-    $defs[-1] = "s6_addr=s6_addr8"
-  end
-  if ipv6 == "kame" && have_struct_member("struct in6_addr", "s6_addr32", headers)
-    $defs[-1] = "-DFAITH"
+    $defs[-1] = "-DHAVE_ADDR8"
   end
   $CPPFLAGS="-I. "+$CPPFLAGS
   $objs += ["getaddrinfo.#{$OBJEXT}"]
@@ -299,6 +255,8 @@ end
 have_header("sys/un.h")
 have_header("sys/uio.h")
 
+$distcleanfiles << "constants.h"
+
 if have_func(test_func)
   have_func("hsterror")
   have_func("getipnodebyname") or have_func("gethostbyname2")
