@@ -1,5 +1,5 @@
 /*
-    $Id: strscan.c,v 1.7.2.8 2006/07/26 09:37:00 aamine Exp $
+    $Id: strscan.c 13506 2007-09-24 08:56:24Z nobu $
 
     Copyright (c) 1999-2006 Minero Aoki
 
@@ -8,8 +8,9 @@
     the Ruby License. For details, see the file COPYING.
 */
 
-#include "ruby.h"
-#include "re.h"
+#include "ruby/ruby.h"
+#include "ruby/re.h"
+#include "ruby/encoding.h"
 
 #define STRSCAN_VERSION "0.7.0"
 
@@ -41,13 +42,13 @@ struct strscanner
 #define MATCHED(s)             (s)->flags |= FLAG_MATCHED
 #define CLEAR_MATCH_STATUS(s)  (s)->flags &= ~FLAG_MATCHED
 
-#define S_PBEG(s)  (RSTRING((s)->str)->ptr)
-#define S_LEN(s)  (RSTRING((s)->str)->len)
+#define S_PBEG(s)  (RSTRING_PTR((s)->str))
+#define S_LEN(s)  (RSTRING_LEN((s)->str))
 #define S_PEND(s)  (S_PBEG(s) + S_LEN(s))
 #define CURPTR(s) (S_PBEG(s) + (s)->curr)
 #define S_RESTLEN(s) (S_LEN(s) - (s)->curr)
 
-#define EOS_P(s) ((s)->curr >= RSTRING(p->str)->len)
+#define EOS_P(s) ((s)->curr >= RSTRING_LEN(p->str))
 
 #define GET_SCANNER(obj,var) do {\
     Data_Get_Struct(obj, struct strscanner, var);\
@@ -62,7 +63,7 @@ static VALUE infect _((VALUE str, struct strscanner *p));
 static VALUE extract_range _((struct strscanner *p, long beg_i, long end_i));
 static VALUE extract_beg_len _((struct strscanner *p, long beg_i, long len));
 
-static void check_strscan _((VALUE obj));
+void check_strscan _((VALUE obj));
 static void strscan_mark _((struct strscanner *p));
 static void strscan_free _((struct strscanner *p));
 static VALUE strscan_s_allocate _((VALUE klass));
@@ -156,7 +157,7 @@ strscan_mark(struct strscanner *p)
 static void
 strscan_free(struct strscanner *p)
 {
-    re_free_registers(&(p->regs));
+    onig_region_free(&(p->regs), 0);
     free(p);
 }
 
@@ -168,7 +169,7 @@ strscan_s_allocate(VALUE klass)
     p = ALLOC(struct strscanner);
     MEMZERO(p, struct strscanner, 1);
     CLEAR_MATCH_STATUS(p);
-    MEMZERO(&(p->regs), struct re_registers, 1);
+    onig_region_init(&(p->regs));
     p->str = Qnil;
     return Data_Wrap_Struct(klass, strscan_mark, strscan_free, p);
 }
@@ -193,7 +194,7 @@ strscan_initialize(int argc, VALUE *argv, VALUE self)
     return self;
 }
 
-static void
+void
 check_strscan(VALUE obj)
 {
     if (TYPE(obj) != T_DATA || RDATA(obj)->dmark != (RUBY_DATA_FUNC)strscan_mark) {
@@ -219,12 +220,13 @@ strscan_init_copy(VALUE vself, VALUE vorig)
     check_strscan(vorig);
     Data_Get_Struct(vorig, struct strscanner, orig);
     if (self != orig) {
-        self->flags = orig->flags;
-        self->str = orig->str;
-        self->prev = orig->prev;
-        self->curr = orig->curr;
-        re_copy_registers(&self->regs, &orig->regs);
+	self->flags = orig->flags;
+	self->str = orig->str;
+	self->prev = orig->prev;
+	self->curr = orig->curr;
+	onig_region_copy(&self->regs, &orig->regs);
     }
+
     return vself;
 }
 
@@ -404,17 +406,15 @@ strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
         return Qnil;
     }
     if (headonly) {
-        ret = re_match(RREGEXP(regex)->ptr,
-                       CURPTR(p), S_RESTLEN(p),
-                       0,
-                       &(p->regs));
+        ret = onig_match(RREGEXP(regex)->ptr, (UChar* )CURPTR(p),
+                         (UChar* )(CURPTR(p) + S_RESTLEN(p)),
+                         (UChar* )CURPTR(p), &(p->regs), ONIG_OPTION_NONE);
     }
     else {
-        ret = re_search(RREGEXP(regex)->ptr,
-                        CURPTR(p), S_RESTLEN(p),
-                        0,
-                        S_RESTLEN(p),
-                        &(p->regs));
+        ret = onig_search(RREGEXP(regex)->ptr,
+                          (UChar* )CURPTR(p), (UChar* )(CURPTR(p) + S_RESTLEN(p)),
+                          (UChar* )CURPTR(p), (UChar* )(CURPTR(p) + S_RESTLEN(p)),
+                          &(p->regs), ONIG_OPTION_NONE);
     }
 
     if (ret == -2) rb_raise(ScanError, "regexp buffer overflow");
@@ -534,7 +534,6 @@ strscan_scan_full(VALUE self, VALUE re, VALUE s, VALUE f)
     return strscan_do_scan(self, re, RTEST(s), RTEST(f), 1);
 }
 
-
 /*
  * call-seq: scan_until(pattern)
  *
@@ -628,26 +627,18 @@ strscan_search_full(VALUE self, VALUE re, VALUE s, VALUE f)
     return strscan_do_scan(self, re, RTEST(s), RTEST(f), 0);
 }
 
-/* DANGEROUS; need to synchronize with regex.c */
 static void
 adjust_registers_to_matched(struct strscanner *p)
 {
-    if (p->regs.allocated == 0) {
-        p->regs.beg = ALLOC_N(int, RE_NREGS);
-        p->regs.end = ALLOC_N(int, RE_NREGS);
-        p->regs.allocated = RE_NREGS;
-    }
-    p->regs.num_regs = 1;
-    p->regs.beg[0] = 0;
-    p->regs.end[0] = p->curr - p->prev;
+    onig_region_clear(&(p->regs));
+    onig_region_set(&(p->regs), 0, 0, p->curr - p->prev);
 }
 
 /*
  * Scans one character and returns it.
- * This method is multi-byte character sensitive.
- * See also #get_byte.
+ * This method is multibyte character sensitive.
  *
- *   s = StringScanner.new('ab')
+ *   s = StringScanner.new("ab")
  *   s.getch           # => "a"
  *   s.getch           # => "b"
  *   s.getch           # => nil
@@ -667,7 +658,8 @@ strscan_getch(VALUE self)
     CLEAR_MATCH_STATUS(p);
     if (EOS_P(p))
         return Qnil;
-    len = mbclen(*CURPTR(p));
+
+    len = rb_enc_mbclen(CURPTR(p), S_PEND(p), rb_enc_get(p->str));
     if (p->curr + len > S_LEN(p)) {
         len = S_LEN(p) - p->curr;
     }
@@ -681,14 +673,15 @@ strscan_getch(VALUE self)
 
 /*
  * Scans one byte and returns it.
- * This method is NOT multi-byte character sensitive.
- * See also #getch.
+ * This method is not multibyte character sensitive.
+ * See also: #getch.
  *
  *   s = StringScanner.new('ab')
  *   s.get_byte         # => "a"
  *   s.get_byte         # => "b"
  *   s.get_byte         # => nil
  *
+ *   $KCODE = 'EUC'
  *   s = StringScanner.new("\244\242")
  *   s.get_byte         # => "\244"
  *   s.get_byte         # => "\242"
@@ -701,9 +694,9 @@ strscan_get_byte(VALUE self)
 
     GET_SCANNER(self, p);
     CLEAR_MATCH_STATUS(p);
-    if (EOS_P(p)) {
+    if (EOS_P(p))
         return Qnil;
-    }
+
     p->prev = p->curr;
     p->curr++;
     MATCHED(p);
@@ -741,13 +734,13 @@ strscan_peek(VALUE self, VALUE vlen)
     long len;
 
     GET_SCANNER(self, p);
+
     len = NUM2LONG(vlen);
-    if (EOS_P(p)) {
+    if (EOS_P(p))
         return infect(rb_str_new("", 0), p);
-    }
-    if (p->curr + len > S_LEN(p)) {
+
+    if (p->curr + len > S_LEN(p))
         len = S_LEN(p) - p->curr;
-    }
     return extract_beg_len(p, p->curr, len);
 }
 
@@ -771,7 +764,7 @@ strscan_peep(VALUE self, VALUE vlen)
  *   s.unscan
  *   s.scan(/../)         # => "te"
  *   s.scan(/\d/)         # => nil
- *   s.unscan             # ScanError: unscan failed: previous match had failed
+ *   s.unscan             # ScanError: unscan failed: previous match record not exist
  */
 static VALUE
 strscan_unscan(VALUE self)
@@ -779,9 +772,8 @@ strscan_unscan(VALUE self)
     struct strscanner *p;
 
     GET_SCANNER(self, p);
-    if (! MATCHED_P(p)) {
-        rb_raise(ScanError, "unscan failed: previous match had failed");
-    }
+    if (! MATCHED_P(p))
+        rb_raise(ScanError, "unscan failed: previous match record not exist");
     p->curr = p->prev;
     CLEAR_MATCH_STATUS(p);
     return self;
@@ -889,7 +881,6 @@ strscan_matched(VALUE self)
 
     GET_SCANNER(self, p);
     if (! MATCHED_P(p)) return Qnil;
-
     return extract_range(p, p->prev + p->regs.beg[0],
                             p->prev + p->regs.end[0]);
 }
@@ -911,7 +902,6 @@ strscan_matched_size(VALUE self)
 
     GET_SCANNER(self, p);
     if (! MATCHED_P(p)) return Qnil;
-
     return INT2NUM(p->regs.end[0] - p->regs.beg[0]);
 }
 
@@ -976,7 +966,6 @@ strscan_pre_match(VALUE self)
 
     GET_SCANNER(self, p);
     if (! MATCHED_P(p)) return Qnil;
-
     return extract_range(p, 0, p->prev + p->regs.beg[0]);
 }
 
@@ -996,7 +985,6 @@ strscan_post_match(VALUE self)
 
     GET_SCANNER(self, p);
     if (! MATCHED_P(p)) return Qnil;
-
     return extract_range(p, p->prev + p->regs.end[0], S_LEN(p));
 }
 
@@ -1029,7 +1017,6 @@ strscan_rest_size(VALUE self)
     if (EOS_P(p)) {
         return INT2FIX(0);
     }
-
     i = S_LEN(p) - p->curr;
     return INT2FIX(i);
 }
@@ -1083,7 +1070,7 @@ strscan_inspect(VALUE self)
         len = snprintf(buf, BUFSIZE, "#<%s %ld/%ld @ %s>",
                        rb_class2name(CLASS_OF(self)),
                        p->curr, S_LEN(p),
-                       RSTRING(b)->ptr);
+                       RSTRING_PTR(b));
         return infect(rb_str_new(buf, len), p);
     }
     a = inspect1(p);
@@ -1091,8 +1078,8 @@ strscan_inspect(VALUE self)
     len = snprintf(buf, BUFSIZE, "#<%s %ld/%ld %s @ %s>",
                    rb_class2name(CLASS_OF(self)),
                    p->curr, S_LEN(p),
-                   RSTRING(a)->ptr,
-                   RSTRING(b)->ptr);
+                   RSTRING_PTR(a),
+                   RSTRING_PTR(b));
     return infect(rb_str_new(buf, len), p);
 }
 
@@ -1243,7 +1230,7 @@ inspect2(struct strscanner *p)
  * There are aliases to several of the methods.
  */
 void
-Init_strscan(void)
+Init_strscan()
 {
     ID id_scanerr = rb_intern("ScanError");
     VALUE tmp;
@@ -1256,7 +1243,7 @@ Init_strscan(void)
     tmp = rb_str_new2(STRSCAN_VERSION);
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Version"), tmp);
-    tmp = rb_str_new2("$Id: strscan.c,v 1.7.2.8 2006/07/26 09:37:00 aamine Exp $");
+    tmp = rb_str_new2("$Id: strscan.c 13506 2007-09-24 08:56:24Z nobu $");
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Id"), tmp);
     
