@@ -1,6 +1,6 @@
 #
 #               tk/text.rb - Tk text classes
-#                       $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
+#                       $Date: 2007-12-21 17:57:35 +0900 (Fri, 21 Dec 2007) $
 #                       by Yukihiro Matsumoto <matz@caelum.co.jp>
 require 'tk'
 require 'tk/itemfont'
@@ -60,7 +60,7 @@ module TkTextTagConfig
   private :itemconfiginfo, :current_itemconfiginfo
 end
 
-class TkText<TkTextWin
+class Tk::Text<TkTextWin
   ItemConfCMD = ['tag'.freeze, 'configure'.freeze].freeze
   #include TkTreatTextTagFont
   include TkTextTagConfig
@@ -1183,6 +1183,11 @@ class TkText<TkTextWin
   end
 
   def _ktext_length(txt)
+    if RUBY_VERSION < '1.9.0' ### !!!!!!!!!!!!!
+      return txt.length
+    end
+    ###########################
+
     if $KCODE !~ /n/i
       return txt.gsub(/[^\Wa-zA-Z_\d]/, ' ').length
     end
@@ -1527,6 +1532,9 @@ class TkText<TkTextWin
   end
 end
 
+TkText = Tk::Text unless Object.const_defined? :TkText
+
+
 #######################################
 
 class TkText::Peer < TkText
