@@ -55,8 +55,6 @@ $goldberg_demo = TkToplevel.new {|w|
 #  positionWindow(w)
 }
 
-base_frame = TkFrame.new($goldberg_demo).pack(:fill=>:both, :expand=>true)
-
 =begin
 # label
 msg = TkLabel.new($goldberg_demo) {
@@ -179,8 +177,7 @@ class TkGoldberg_Demo
     do_ctrl_frame
     do_detail_frame
 
-    # msg = TkLabel.new(@parent, :bg=>@C['bg'], :fg=>'white') {
-    msg = Tk::Label.new(@parent, :bg=>@C['bg'], :fg=>'white') {
+    msg = TkLabel.new(@parent, :bg=>@C['bg'], :fg=>'white') {
       font 'Arial 10'
       wraplength 600
       justify 'left'
@@ -190,8 +187,7 @@ class TkGoldberg_Demo
 
     frame = TkFrame.new(@parent, :bg=>@C['bg'])
 
-    # TkButton.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
-    Tk::Button.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
+    TkButton.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
       text 'Ĥ'
       command proc{
         tmppath = $goldberg_demo
@@ -200,14 +196,12 @@ class TkGoldberg_Demo
       }
     }.pack('side'=>'left')
 
-    # TkButton.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
-    Tk::Button.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
+    TkButton.new(frame, :bg=>@C['bg'], :activebackground=>@C['bg']) {
       text 'ɻ'
       command proc{showCode 'goldberg'}
     }.pack('side'=>'left', 'padx'=>5)
 
-    # @show = TkButton.new(frame, :text=>'>>', :command=>proc{show_ctrl}, 
-    @show = Tk::Button.new(frame, :text=>'>>', :command=>proc{show_ctrl}, 
+    @show = TkButton.new(frame, :text=>'>>', :command=>proc{show_ctrl}, 
                          :bg=>@C['bg'], :activebackground=>@C['bg'])
     @show.pack('side'=>'left')
     frame.place(:in=>@canvas, :relx=>1, :rely=>0, :anchor=>:ne)
@@ -216,11 +210,10 @@ class TkGoldberg_Demo
   end
 
   def do_ctrl_frame
-    @start = Tk::Button.new(@parent, :text=>'Start', :bd=>6, 
+    @start = TkButton.new(@parent, :text=>'Start', :bd=>6, 
                           :command=>proc{do_button(0)})
-    if font = @start['font']
-      @start.font(font.weight('bold'))
-    end
+    @start.font(@start['font'].weight('bold'))
+    font = @start['font']
 
     @pause = TkCheckbutton.new(@parent, :text=>'Pause', :font=>font, 
                                :command=>proc{do_button(1)}, :relief=>:raised, 
@@ -2008,4 +2001,4 @@ class TkGoldberg_Demo
   end
 end
 
-TkGoldberg_Demo.new(base_frame)
+TkGoldberg_Demo.new($goldberg_demo)
