@@ -16,10 +16,8 @@ $image2_demo = TkToplevel.new {|w|
   positionWindow(w)
 }
 
-base_frame = TkFrame.new($image2_demo).pack(:fill=>:both, :expand=>true)
-
 # label 
-msg = TkLabel.new(base_frame) {
+msg = TkLabel.new($image2_demo) {
   font $font
   wraplength '4i'
   justify 'left'
@@ -28,7 +26,7 @@ msg = TkLabel.new(base_frame) {
 msg.pack('side'=>'top')
 
 # frame 
-TkFrame.new(base_frame) {|frame|
+TkFrame.new($image2_demo) {|frame|
   TkButton.new(frame) {
     #text 'λ'
     text 'Ĥ'
@@ -53,21 +51,21 @@ $dirName = TkVariable.new([$demo_dir,'..','images'].join(File::Separator))
 $image2a = TkPhotoImage.new
 
 # ե̾
-TkLabel.new(base_frame, 'text'=>'ǥ쥯ȥ:')\
+TkLabel.new($image2_demo, 'text'=>'ǥ쥯ȥ:')\
 .pack('side'=>'top', 'anchor'=>'w')
 
-image2_e = TkEntry.new(base_frame) {
+image2_e = TkEntry.new($image2_demo) {
   width 30
   textvariable $dirName
 }.pack('side'=>'top', 'anchor'=>'w')
 
-TkFrame.new(base_frame, 'height'=>'3m', 'width'=>20)\
+TkFrame.new($image2_demo, 'height'=>'3m', 'width'=>20)\
 .pack('side'=>'top', 'anchor'=>'w')
 
-TkLabel.new(base_frame, 'text'=>'ե:')\
+TkLabel.new($image2_demo, 'text'=>'ե:')\
 .pack('side'=>'top', 'anchor'=>'w')
 
-TkFrame.new(base_frame){|w|
+TkFrame.new($image2_demo){|w|
   s = TkScrollbar.new(w)
   l = TkListbox.new(w) {
     width 20
@@ -86,10 +84,9 @@ TkFrame.new(base_frame){|w|
 }.pack('side'=>'top', 'anchor'=>'w')
 
 # image 
-[ TkFrame.new(base_frame, 'height'=>'3m', 'width'=>20),
-  TkLabel.new(base_frame, 'text'=>':'),
-  # TkLabel.new(base_frame, 'image'=>$image2a)
-  Tk::Label.new(base_frame, 'image'=>$image2a)
+[ TkFrame.new($image2_demo, 'height'=>'3m', 'width'=>20),
+  TkLabel.new($image2_demo, 'text'=>':'),
+  TkLabel.new($image2_demo, 'image'=>$image2a)
 ].each{|w| w.pack('side'=>'top', 'anchor'=>'w')}
 
 # ᥽å
