@@ -375,14 +375,7 @@ class Options
       @css = nil
       @webcvs = nil
 
-      @charset = case $KCODE
-                 when /^S/i
-                   'Shift_JIS'
-                 when /^E/i
-                   'EUC-JP'
-                 else
-                   'iso-8859-1'
-                 end
+      @charset = 'iso-8859-1'
 
       accessors = []
 
@@ -393,7 +386,7 @@ class Options
 	case opt
         when "--all"           then @show_all      = true
         when "--charset"       then @charset       = arg
-        when "--debug"         then $DEBUG         = true
+        when "--debug"         then $DEBUG_RDOC    = true
         when "--exclude"       then @exclude       << Regexp.new(arg)
         when "--inline-source" then @inline_source = true
         when "--line-numbers"  then @include_line_numbers = true
