@@ -2,7 +2,7 @@
 
 exit if defined?(CROSS_COMPILING)
 load './rbconfig.rb'
-include Config
+include RbConfig
 
 ruby = "./#{CONFIG['ruby_install_name']}#{CONFIG['EXEEXT']}"
 unless File.exist? ruby
@@ -37,13 +37,13 @@ $stderr.reopen($stdout)
 error = ''
 
 srcdir = File.dirname(__FILE__)
-`#{ruby} -I#{srcdir}/lib #{srcdir}/sample/test.rb`.each do |line|
+`#{ruby} -I#{srcdir}/lib #{srcdir}/sample/test.rb`.each_line do |line|
   if line =~ /^end of test/
-    print "test succeeded\n"
-    exit 0
+    print "\ntest succeeded\n"
+    exit true
   end
-  error << line if line =~ %r:^(sample/test.rb|not):
+  error << line if %r:^(sample/test.rb|not): =~ line
 end
 print error
 print "test failed\n"
-exit 1
+exit false
