@@ -247,19 +247,6 @@ class TestStringScanner < Test::Unit::TestCase
     s = StringScanner.new("")
     assert_equal "", s.scan(//)
     assert_equal "", s.scan(//)
-
-    # [ruby-dev:29914]
-    %w( NONE EUC SJIS UTF8 ).each do |kcode|
-      begin
-        $KCODE = kcode
-        assert_equal "a", StringScanner.new("a:b").scan(/[^\x01\:]+/n)
-        assert_equal "a", StringScanner.new("a:b").scan(/[^\x01\:]+/e)
-        assert_equal "a", StringScanner.new("a:b").scan(/[^\x01\:]+/s)
-        assert_equal "a", StringScanner.new("a:b").scan(/[^\x01\:]+/u)
-      ensure
-        $KCODE = 'NONE'
-      end
-    end
   end
 
   def test_skip
@@ -301,15 +288,9 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal true, s.getch.tainted?
     assert_nil s.getch
 
-    kc_backup = $KCODE
-    begin
-      $KCODE = 'EUC'
-      s = StringScanner.new("\244\242")
-      assert_equal "\244\242", s.getch
-      assert_nil s.getch
-    ensure
-      $KCODE = kc_backup
-    end
+    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
+    assert_equal "\244\242", s.getch
+    assert_nil s.getch
 
     s = StringScanner.new('test')
     s.scan(/te/)
@@ -335,16 +316,10 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal true, s.get_byte.tainted?
     assert_nil s.get_byte
 
-    kc_backup = $KCODE
-    begin
-      $KCODE = 'EUC'
-      s = StringScanner.new("\244\242")
-      assert_equal "\244", s.get_byte
-      assert_equal "\242", s.get_byte
-      assert_nil s.get_byte
-    ensure
-      $KCODE = kc_backup
-    end
+    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
+    assert_equal "\244", s.get_byte
+    assert_equal "\242", s.get_byte
+    assert_nil s.get_byte
 
     s = StringScanner.new('test')
     s.scan(/te/)
@@ -437,16 +412,9 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil           s[0]
 
 
-    kc_backup = $KCODE
-    begin
-      $KCODE = 'EUC'
-      s = StringScanner.new("\244\242")
-      s.getch
-      assert_equal "\244\242", s[0]
-    ensure
-      $KCODE = kc_backup
-    end
-
+    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
+    s.getch
+    assert_equal "\244\242", s[0]
 
     str = 'test'
     str.taint
